/*********************************************************************/
/* test.c	(v. 0.1)					     */
/* Asymmetric login key tester for neblo			     */
/* Copyright (C) 2003 - Paolo Gianrossi <paolino@member.fsf.org>     */
/* 								     */
/* This program is free software; you can redistribute it and/or     */
/* modify it under the terms of the GNU General Public License as    */
/* published by the Free Software Foundation; either version 2 of    */
/* the License, or (at your option) any later version.		     */
/*								     */
/* This program is distributed in the hope that it will be useful,   */
/* but WITHOUT ANY WARRANTY; without even the implied warranty of    */
/* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU */
/* General Public License for more details.			     */
/*								     */
/* You should have received a copy of the GNU General Public License */
/* along with this program; if not, write to the Free Software	     */
/* Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA	     */
/* 02111-1307, USA.						     */
/*********************************************************************/

#include <openssl/rsa.h>
#include <openssl/rand.h> 

#include <stdio.h> 
#include <string.h> 
#include <stdlib.h> 
#include <unistd.h>

#include <neblo_secret.h>

#define DEFAULTMSG "pippo123"

int main(int argc, char** argv) 
{

  RSA *publicKey=RSA_new(); 
  RSA *secretKey=RSA_new();
  
  unsigned char *msg= DEFAULTMSG;
   
  unsigned char *cipher; 
  unsigned char *clearMsg;
  
  char *pubfile=DEFAULTPUBKEYFILE;
  char *secfile=DEFAULTSECKEYFILE;
  
  int long_msg; 
  
  char fieldStr[MAXBYTEKEYSIZE-1];
  FILE* pf;
  FILE *sf;

  int rc;
  int found_h=0;
  int found_k=0;
  int found_m=0;
  
    /*cmdline opts */

  for (;;) {
    while ((rc = getopt(argc,argv,"k:m:h")) != 'k' \
	   && rc != 'm' &&rc!='h' && rc != -1);
    
    if (rc == -1)
      break;
    if (rc == 'h' && !found_h) /* "h" */ {
      found_h = 1;
      fprintf(stdout,
	      "%s\n"
	      "Validation test for a neblo RSA key pair.\n"
	      "It encrypts and then decrypts a message and displays it.\n"
	      "OPTIONS:\n"
	      "-k :\tSelect the keypair name. It defaults to %s.\n"
	      "-m :\tSelect the message to encrypt. Default is \"%s\".\n"
	      "-h :\tDisplays this very help and quits.\n",
	      argv[0], DEFAULTKEYFILE, DEFAULTMSG);
      exit(0);
    
    }
    if (rc == 'm' && !found_m) /* "m" */ {
      found_m = 1;
      msg=(char*)malloc((strlen(optarg)+1)*sizeof(char));
      strncpy(msg,optarg,strlen(optarg));
      strcat(msg,"\0");
    }
    if (rc =='k' && !found_k) /* "k" */ {
      found_k = 1;
      pubfile=(char*)malloc((strlen(optarg)+5)*sizeof(char));
      secfile=(char*)malloc((strlen(optarg)+5)*sizeof(char));
      strncpy(pubfile,optarg,strlen(optarg));
      strcat(pubfile,".pk\0");
      strncpy(secfile,optarg,strlen(optarg));
      strcat(secfile,".sk\0");
    }
  }

  
  pf=fopen(pubfile, "r");	/* public key file */
  sf=fopen(secfile, "r");	/* secret key file */
  if( !pf || !sf){
    fprintf(stderr, "ERROR! Couldn't open key files.\nAborting!\n");
    exit(1);
  }

  _get_public_key_(pf, publicKey, fieldStr);
  
  fprintf(stdout, "--Public Key Begin--\n");
  RSA_print_fp(stdout, publicKey, 0); 
  fprintf(stdout, "--Public Key End--\n\n");
  
  _get_private_key_(sf, secretKey, fieldStr);
    
  /* Encriptamos el msg */
  
  cipher= (unsigned char *) malloc (RSA_size(publicKey));
  long_msg= (strlen(msg)+1)*sizeof(char);
  RSA_public_encrypt(long_msg, msg, cipher, publicKey, \
		     RSA_PKCS1_OAEP_PADDING);
  
  
  /* decrypt msg */
  clearMsg= (unsigned char *) malloc(RSA_size(secretKey)); 
  RSA_private_decrypt(RSA_size(secretKey), cipher, clearMsg, secretKey, \
		      RSA_PKCS1_OAEP_PADDING); 
  
  
  fprintf(stdout, "The original message: %s\n", msg);
  fprintf(stdout, "The message was encrypted.\n");
  fprintf(stdout, "Again the ol'message: %s\n", clearMsg);
  
  RSA_free(publicKey);
  free(cipher);
  free(clearMsg);

  return 0;

} 
